"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.isVersionStored = exports.redirectTo = void 0;
const client_s3_1 = require("@aws-sdk/client-s3");
/**
 * The function `redirectTo` is used to create a redirect response with a specified URL.
 * @param {string} url - The `url` parameter is a string that represents the URL to which you want to
 * redirect the user.
 * @param {any} callback - The `callback` parameter is a function that is used to return the response
 * to the caller. It takes two arguments: an error object (if any) and the response object. In this
 * case, the response object is an HTTP response with a status code of 302 (Redirect) and a `
 * @returns a callback function with two arguments: null and an object representing a response.
 */
const redirectTo = (url, callback) => {
    const response = {
        status: 302,
        statusDescription: 'Redirect',
        headers: {
            location: [
                {
                    key: 'Location',
                    value: url,
                },
            ],
            'cache-control': [
                {
                    key: 'Cache-Control',
                    value: 'public, max-age=600, stale-while-revalidate=2592000', // Serve cached content up to 30 days old while revalidating it after 10 minutes
                },
            ],
        },
    };
    return callback(null, response);
};
exports.redirectTo = redirectTo;
const isVersionStored = (s3Client, bucket, key) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const command = new client_s3_1.HeadObjectCommand({
            Bucket: bucket,
            Key: key,
        });
        // will throw error if it's not found
        yield s3Client.send(command);
        return true;
    }
    catch (error) {
        console.warn(`${key} is not stored yet`, error);
        return false;
    }
});
exports.isVersionStored = isVersionStored;
